/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * $Author$
 * $Date  $
 * $RCSfile: drv_hdmi.h,v $
 * $Revision$
 *
 *---------------------------------------------------------------------------*/
#ifndef _DRV_HDMI2_H_
#define _DRV_HDMI2_H_

#include "typedef.h"
#include "x_timer.h"

#define HDMI_RX_NEW_AVI (1<<16)
#define HDMI_RX_NEW_SP (1<<17)
#define HDMI_RX_NEW_AUD (1<<18)
#define HDMI_RX_NEW_MPEG (1<<19)
#define HDMI_RX_NEW_UNREC (1<<20)
#define HDMI_RX_NEW_CP (1<<23)
#define HDMI_RX_INVALID_AVI (1<<28)

#define HDMI_RX_NEW_HDR_0x86 (1<<6)
#define HDMI_RX_NEW_HDR_0x87 (1<<7)

#define HDMI_RX_AAC_MUTE (1<<6)
#define HDMI_RX_NEW_ACP (1<<10)
#define HDMI_RX_NEW_VSIF (1<<22)

#define HDMI_RX_NEW_ISRC1 (1<<2)
#define HDMI_RX_NEW_ISRC2 (1<<3)
#define HDMI_RX_NEW_GCP   (1<<4)
#define HDMI_RX_NEW_HF_VSIF (1<<9)

#define HDMI_RX_NEW_DOLBY_EDR_MD    (1<<1)
#define HDMI_RX_DOLBY_EDR_CRC_ERROR (1<<2)

#define HDMI_RX_HDCP2X_INT_AUTH_DONE      (1)
#define HDMI_RX_HDCP2X_INT_AKE_SENT_RCVD  (1<<18)

#define HDMI_RST_ALL    1
#define HDMI_RST_EQ        2
#define HDMI_RST_DEEPCOLOR    3
#define HDMI_RST_FIXEQ        4
#define HDMI_RST_RTCK        5
#define HDMI_RST_DIGITALPHY 6

#define HDMI_AUD_Length_Unknown      0
#define HDMI_AUD_Length_16bits      1
#define HDMI_AUD_Length_17bits      2
#define HDMI_AUD_Length_18bits      3
#define HDMI_AUD_Length_19bits      4
#define HDMI_AUD_Length_20bits      5
#define HDMI_AUD_Length_21bits      6
#define HDMI_AUD_Length_22bits      7
#define HDMI_AUD_Length_23bits      8
#define HDMI_AUD_Length_24bits      9

#define HDMI_ENABLE_EQ_TRACKING_6G 1

#define HDMI_HDR_METADATA_LENGTH 24
#define HDMI_INVALID_HDR_EOTF_VALUE 0xff

#define HDMI_DOLBY_EDR_MAX_BODY_LENGTH 482 //Support total 482byte metadata body = (119 + 121*3)
#define HDMI_DOLBY_EDR_MAX_METADATA_BUFFER 1000 //define max metadat buffer for debug purpose

enum
{
    HDMI_3D_Structure_FramePacking=0,
    HDMI_3D_Structure_FieldAlternative,
    HDMI_3D_Structure_LineAlternative,
    HDMI_3D_Structure_SideBySideFull,
    HDMI_3D_Structure_LDepth,
    HDMI_3D_Structure_LDepthGraph,
    HDMI_3D_Structure_TopBottom,
    HDMI_3D_Structure_SideBySideHalf=8,
    HDMI_3D_Structure_Unknow
};

enum
{
    HDMI_NOSIGNAL_STATE=0,
    HDMI_WAIT_CONNECT_STATE,
    HDMI_WAIT_STABLE_STATE,
    HDMI_STABLE_STATE
};
typedef enum
{
    HDMI_RX_MODE_DVI,
    HDMI_RX_MODE_HDMI1X,
    HDMI_RX_MODE_HDMI2X,
    HDMI_RX_MODE_MHL
}HdmiRxMode_t;

typedef enum eHDMI_SWITCH_NO {
    HDMI_SWITCH_INIT = 0,
    HDMI_SWITCH_1,
    HDMI_SWITCH_2,
    HDMI_SWITCH_3,
    HDMI_SWITCH_4,
    HDMI_SWITCH_MAX
}E_HDMI_SWITCH_NUM;

typedef enum
{
    HDMI_RX_PORT_Init=0,
    HDMI_RX_PORT_1,
    HDMI_RX_PORT_2,
    HDMI_RX_PORT_3,
    HDMI_RX_PORT_4,
    HDMI_RX_PORT_MAX
}HdmiRxPort;

#define HDMI_MAX_PHYCIAL_PORT_NUM HDMI_SWITCH_4 //define it for each SOC

typedef enum
{
    HDMI_RX_INPUT_TYPE_NONE,
    HDMI_RX_INPUT_TYPE_HDMI,
    HDMI_RX_INPUT_TYPE_MHL,
}HdmiRxInputType_t;


typedef enum
{
    HDMI_RX_CLRSPC_UNKNOW,
    HDMI_RX_CLRSPC_YC444_601,
    HDMI_RX_CLRSPC_YC422_601,
    HDMI_RX_CLRSPC_YC420_601,

    HDMI_RX_CLRSPC_YC444_709,
    HDMI_RX_CLRSPC_YC422_709,
    HDMI_RX_CLRSPC_YC420_709,

    HDMI_RX_CLRSPC_XVYC444_601,
    HDMI_RX_CLRSPC_XVYC422_601,
    HDMI_RX_CLRSPC_XVYC420_601,

    HDMI_RX_CLRSPC_XVYC444_709,
    HDMI_RX_CLRSPC_XVYC422_709,
    HDMI_RX_CLRSPC_XVYC420_709,

    HDMI_RX_CLRSPC_sYCC444_601,
    HDMI_RX_CLRSPC_sYCC422_601,
    HDMI_RX_CLRSPC_sYCC420_601,

    HDMI_RX_CLRSPC_Adobe_YCC444_601,
    HDMI_RX_CLRSPC_Adobe_YCC422_601,
    HDMI_RX_CLRSPC_Adobe_YCC420_601,

    HDMI_RX_CLRSPC_RGB,
    HDMI_RX_CLRSPC_Adobe_RGB,

    HDMI_RX_CLRSPC_BT_2020_RGB_non_const_luminous,
    HDMI_RX_CLRSPC_BT_2020_RGB_const_luminous,

    HDMI_RX_CLRSPC_BT_2020_YCC444_non_const_luminous,
    HDMI_RX_CLRSPC_BT_2020_YCC422_non_const_luminous,
    HDMI_RX_CLRSPC_BT_2020_YCC420_non_const_luminous,

    HDMI_RX_CLRSPC_BT_2020_YCC444_const_luminous,
    HDMI_RX_CLRSPC_BT_2020_YCC422_const_luminous,
    HDMI_RX_CLRSPC_BT_2020_YCC420_const_luminous

}HdmiRxClrSpc_t;

typedef enum
{
    HDMI_RX_BIT_DEPTH_8_BIT,
    HDMI_RX_BIT_DEPTH_10_BIT,
    HDMI_RX_BIT_DEPTH_12_BIT,
    HDMI_RX_BIT_DEPTH_16_BIT
}HdmiRxBitDepth_t;

typedef enum
{
    HDMI_RX_RGB_FULL,
    HDMI_RX_RGB_LIMT,
    HDMI_RX_YCC_FULL,
    HDMI_RX_YCC_LIMT
}HdmiRxRange_t;
typedef enum
{
    HDMI_RX_OVERSCAN=1,
    HDMI_RX_UNDERSCAN
}HdmiRxScanInfo_t;

typedef enum
{
    HDMI_RX_ASPECT_RATIO_4_3=1,
    HDMI_RX_ASPECT_RATIO_16_9
}HdmiRxAspectRatio_t;

typedef enum
{
    HDMI_RX_AFD_SAME=8,
    HDMI_RX_AFD_4_3,
    HDMI_RX_AFD_16_9,
    HDMI_RX_AFD_14_9
}HdmiRxAFDInfo_t;

typedef enum
{
    HDMI_RX_ITC_GRAPHICS=0,
    HDMI_RX_ITC_PHOTO,
    HDMI_RX_ITC_CINEMA,
    HDMI_RX_ITC_GAME
}HdmiRxITCInfo_t;

typedef enum
{
    HDMI_RX_HDCP_LINK_VER_NONE,
    HDMI_RX_HDCP_LINK_VER_HDCP1X,
    HDMI_RX_HDCP_LINK_VER_HDCP22,
}HdmiRxHdcpLinkVer_t;

typedef enum
{
    HDMI_RX_HDCP2X_AUTH_OFF=0,
    HDMI_RX_HDCP2X_AUTH_AKE_SENT_RCVD,
    HDMI_RX_HDCP2X_AUTH_AUTH_DONE
}HDMI_RX_HDCP2X_AUTH_STATUS_T;

typedef enum
{
    PROM_HDCP2X_STA_IDLE                    = 0x00,
    PROM_HDCP2X_STA_AUTH_DONE               = 0x2B,
    PROM_HDCP2X_STA_CERT_TIME_OUT           = 0x30,
    PROM_HDCP2X_STA_CERT_FAIL               = 0x31,
    PROM_HDCP2X_STA_H_FAIL                  = 0x32,
    PROM_HDCP2X_STA_L_FAIL                  = 0x33,
    PROM_HDCP2X_STA_V_FAIL                  = 0x34,
    PROM_HDCP2X_STA_MSG_WRONG               = 0x35,
    PROM_HDCP2X_STA_MSG_TIMEOUT             = 0x36,
    PROM_HDCP2X_STA_MSG_WRONG_HEADER        = 0x37,
    PROM_HDCP2X_STA_L_TIMEOUT               = 0x38,
    PROM_HDCP2X_STA_H_TIMEOUT               = 0x39,
    PROM_HDCP2X_STA_V_TIMEOUT               = 0x3A,
    PROM_HDCP2X_STA_SEQ_NUM_NOT_INCREASE    = 0x3B,
    PROM_HDCP2X_STA_SEQ_NUM_ROLL_OVER       = 0x3C,
    PROM_HDCP2X_STA_SEQ_NUM_V_NOT_CLEARED   = 0x3E,
    PROM_HDCP2X_STA_HPD_LOW                 = 0x40
}PROM_8051_HDCP2X_RX_STATUS_T;

typedef enum
{
    HDMI_RX_HDCP1_STATUS_OFF =0x0,
    HDMI_RX_HDCP1_STATUS_AUTH_DONE =0x1,
    HDMI_RX_HDCP1_STATUS_AUTH_FAIL =0x2,
    HDMI_RX_HDCP1_STATUS_DECRYPT_SUCCESS=0x4,
    HDMI_RX_HDCP1_STATUS_DECRYPT_FAIL=0x8
}HdmiRxHdcp1Status_t;

typedef enum
{
    HDMI_RX_AUDIO_FS_24KHZ,
    HDMI_RX_AUDIO_FS_32KHZ,
    HDMI_RX_AUDIO_FS_44_1KHZ,
    HDMI_RX_AUDIO_FS_48KHZ,
    HDMI_RX_AUDIO_FS_88_2KHZ,
    HDMI_RX_AUDIO_FS_96KHZ,
    HDMI_RX_AUDIO_FS_176_4KHZ,
    HDMI_RX_AUDIO_FS_192KHZ,
    HDMI_RX_AUDIO_FS_768KHZ
}HdmiRxAudioFs_t;

typedef enum
{
    HDMI_TMDS_CLOCK_UNKNOW,
    HDMI_TMDS_CLOCK_25,
    HDMI_TMDS_CLOCK_25_35,
    HDMI_TMDS_CLOCK_35_40,
    HDMI_TMDS_CLOCK_40_50,
    HDMI_TMDS_CLOCK_50_80,
    HDMI_TMDS_CLOCK_80_100,
    HDMI_TMDS_CLOCK_100_120,
    HDMI_TMDS_CLOCK_120_140,
    HDMI_TMDS_CLOCK_140_200,
    HDMI_TMDS_CLOCK_200_300
}HdmiTmdsClkRate_t;

typedef enum
{
    MHL_TMDS_CLOCK_UNKNOW,
//    MHL_TMDS_CLOCK_25_37,
    MHL_TMDS_CLOCK_25_35,
    MHL_TMDS_CLOCK_35_37,
//    MHL_TMDS_CLOCK_37_50,
    MHL_TMDS_CLOCK_37_40,
    MHL_TMDS_CLOCK_40_50,
    MHL_TMDS_CLOCK_50_60,
    MHL_TMDS_CLOCK_60_67,
    MHL_TMDS_CLOCK_67_75
}MhlTmdsClkRate_t;


typedef enum
{
    HDMI_AUDIO_FIFO_NORMAL= 0x1,
    HDMI_AUDIO_FIFO_UNDERRUN= 0x2,
    HDMI_AUDIO_FIFO_OVERRUN= 0x4
}HdmiAudFifoStatus_t;

typedef enum
{
    HDMI_TMDS_DATA_CHANNEL_0,
    HDMI_TMDS_DATA_CHANNEL_1,
    HDMI_TMDS_DATA_CHANNEL_2
}HdmiTmdsDataChannel_t;

typedef enum
{
    HDMI_INFO_FRAME_UNKNOWN,
    HDMI_INFO_FRAME_ID_AVI,
    HDMI_INFO_FRAME_ID_SPD,
    HDMI_INFO_FRAME_ID_AUD,
    HDMI_INFO_FRAME_ID_MPEG,
    HDMI_INFO_FRAME_ID_UNREC,
    HDMI_INFO_FRAME_ID_ACP,
    HDMI_INFO_FRAME_ID_VSI,
    HDMI_INFO_FRAME_ID_ISRC0,
    HDMI_INFO_FRAME_ID_ISRC1,
    HDMI_INFO_FRAME_ID_GCP,
    HDMI_INFO_FRAME_ID_HFVSI,
    HDMI_INFO_FRAME_ID_HDR_0x86,
    HDMI_INFO_FRAME_ID_HDR_0x87,
    HDMI_INFO_FRAME_ID_EMPPKT1,
    HDMI_INFO_FRAME_ID_DVVSI
}HdmiInfoFrameID_t;


enum
{
    HDMI_SPD_NONE_CHG,
    HDMI_SPD_CLEAR,//no SPD
    HDMI_SPD_CHG_TOUP,
    HDMI_SPD_CHG_TONONUP,
    HDMI_SPD_CHG_TOSV,
    HDMI_SPD_CHG_TONONSV,
};

enum
{
    HDMI_SPD_NONE_STATUS,
    HDMI_SPD_UP_UNDET,//Without specific pattern
    HDMI_SPD_UP_DETED,//With specific pattern
    HDMI_SPD_SV_UNDET,
    HDMI_SPD_SV_DETED,
};

typedef enum {
    HDMI_HDCP1X,
    HDMI_HDCP2X
}HDMI_HDCP_KEY_CHECK_MODE_T;

typedef enum
{
    HDMI_DOLBY_HDR_NONE,
    HDMI_DOLBY_HDR_LOW_LATENCY,
    HDMI_DOLBY_HDR_DETECT_STANDARD
}E_HDMI_DOLBY_HDR_STATE;

typedef struct
{
    HdmiRxClrSpc_t clrSpc;
    HdmiRxRange_t range;
}HdmiVideoInfo_t;

typedef struct
{
    BOOL fgItc;
    HdmiRxITCInfo_t ItcContent;
}HdmiItcInfo_t;

typedef struct
{
    UINT8 HDMI_3D_Enable;
    UINT8 HDMI_3D_Video_Format;
    UINT8 HDMI_3D_Structure;
    UINT8 HDMI_3D_EXTDATA;
}HdmiVideo3DInfo_t;

typedef struct
{
    UINT8 layout;
    UINT8 dsd;
    UINT8 hbrA;
    HdmiRxAudioFs_t audioFs;
}HdmiRxAudioFormat_t;

typedef struct
{
    UINT8 u1CategoryCode;
    UINT8 u1WordLength;
    UINT8 u1b1;
    UINT8 u1b2;
    UINT8 u1b3;
    BOOL fgCopyRight;
    BOOL fgIsPCM;
}HdmiRxAudioChannelStatus_t;

typedef struct
{
    BOOL fgIsAudMute;
    HdmiRxAudioFormat_t audFormat;
    HdmiRxAudioChannelStatus_t chlStatus;
}HdmiRxAudioInfomation_t;

typedef struct
{
    HdmiInfoFrameID_t u1InfoID;
    UINT8 u1InfoData[32];
}HdmiRxInfo_t;

typedef struct
{
    UINT32 u4Header;
    UINT32 au4VdrName[2];//Vendor Name Chacter VN1 TO VN8, 8 bytes.
    UINT32 au4PrdDspt[4];//Product Description Chacter PD1 TO PD16,16 bytes.
    UINT8 u1SrcInfo;
}HdmiSpdParse_t;

typedef struct
{
    UINT8 u1ScramblingOn;
    UINT8 u1ClkRatio;
}HdmiScdcStatus_t;

typedef struct
{
    UINT8 au1HdrMetadata[HDMI_HDR_METADATA_LENGTH];
}HDMI_HDR_METADATA;

typedef struct
{
    UINT8  u1VRR_EN;
    UINT8  u1M_CONST;
    UINT8  u1FVA_Factor;
    UINT8  u1Base_Vfront;
    UINT8  u1RB;
    UINT16 u2Base_Refesh_Rate;
}HDMI_VRR_Info;

#ifdef CC_HDMI_POH_SUPPORT
typedef enum
{
    HDMI_POH_STATE_IDLE,
    HDMI_POH_STATE_WAIT_CONNECT,
    HDMI_POH_STATE_CONNECTED
}E_HDMI_POH_STATE;

typedef struct _HDMI_POH_CONFIG_T
{
    UINT8 u1PortId; //Driver port ID
    UINT8 fgEnable; //POH enable
    INT32 i4GpioPwrEn; //POH power enable gpio
    INT32 i4GpioPohDet; //POH device detect gpio
    INT32 i4GpioOcpDet; //POH over current detect gpio
}HDMI_POH_CONFIG_T;
#endif //CC_HDMI_POH_SUPPORT

typedef struct
{
    UINT8 _bHdmiCmd;
    UINT8 _bHDMIState;
    UINT8 _bHDMIStateOld;
    UINT8 _bHDMIPower5vStatus;
    BOOL  _fgForce5vOn;
    UINT8 _bHdmiPlug;
    UINT8 _bUnplugFlag;
    UINT8 _bHDMIAvMute;
    UINT16 _bHDMIRxBaseAddr;
    UINT16 _bHDMIAnaBaseAddr;
    UINT8 _bHDMICkdt;
    UINT8 _bHDMIScdt;
    UINT8 _bHDMIScdtLose;
    UINT8 _bHDMIHpdTimerCnt;
    UINT8 _bHDMIHdcp1xGood;
    UINT8 _bHDMIHdcp2xNoExistCnt;
    BOOL _fgIsHDMIHdcp2xExist;
    UINT8 _bHDMIHDCP2xHandlerCnt;
    UINT8 _bHDMIHDCP2xErrCnt;
    BOOL _fgFirstEnterAuthDone;
    UINT8 _bSCDTdelay;
    UINT8 _bHDMIResStableCnt;
    UINT8 _bHDMIResUnStableCnt;
    UINT8 _bIs422;
    UINT8 _bHDMIFastSW;
    UINT8 _bResv1;
    UINT8 _bNoVsiCount;
    UINT8 _bNoSpdCount;
    UINT8 _bNo4KPsCount;
    UINT8 _bHDMIEqTracking;
    UINT8 _bHDMIEqCh0;
    UINT8 _bHDMIEqCh1;
    UINT8 _bHDMIEqCh2;
    UINT16 _bHDMIEqTrackingWaitCnt;
    UINT16 _bHDMIEqTrackingStableCnt;
    UINT32 _dwHDMIClkKhz;
    BOOL _fgHDMI40xMode;
    UINT8 _bSpdStatus;
    UINT8 _bSpdUpStaChg;
    UINT8 _b4KPsStatus;
    UINT8 _bNoAcpCount;
    UINT8 _bHdrEOTF;
    BOOL _fgFirstAnaSetting;
    HAL_TIME_T _rHdrReceiveTime;
    HAL_TIME_T _rHDMIEqTrackingStart;
    HAL_TIME_T _rVsiReceiveTime;
#ifdef CC_HDMI_POH_SUPPORT
    E_HDMI_POH_STATE _ePohState;
#endif
    UINT8 _bDolbyVision;
    UINT8 _bDolbyVisionOld;

    HdmiRxMode_t _HdmiMode;
    HdmiRxInputType_t _ConnectType;
    HdmiRxClrSpc_t _HdmiClrSpc;
    HdmiRxBitDepth_t _HdmiDepth;
    HdmiRxRange_t _HdmiRange;
    HdmiRxScanInfo_t _HdmiScan;
    HdmiRxAspectRatio_t _HdmiAspect;
    HdmiRxAFDInfo_t    _HdmiAFD;
    HdmiItcInfo_t     _HdmiITC;
    HdmiVideo3DInfo_t _Hdmi3DInfo;
    HdmiRxHdcpLinkVer_t _HdcpLinkVer;
    HdmiRxHdcp1Status_t _Hdcp1xState;
    HDMI_RX_HDCP2X_AUTH_STATUS_T _eHdcp2xAuthStatus;

    HdmiScdcStatus_t _ScdcInfo;
    HdmiRxInfo_t _AviInfo;
    HdmiRxInfo_t _SpdInfo;
    HdmiRxInfo_t _AudInfo;
    HdmiRxInfo_t _MpegInfo;
    HdmiRxInfo_t _UnrecInfo;
    HdmiRxInfo_t _Hdr87Info;
    HdmiRxInfo_t _Hdr86Info;
    HdmiRxInfo_t _AcpInfo;
    HdmiRxInfo_t _VsiInfo;
    HdmiRxInfo_t _Isrc0Info;
    HdmiRxInfo_t _Isrc1Info;
    HdmiRxInfo_t _GcpInfo;
    HdmiRxInfo_t _HfVsiInfo;
    HdmiRxAudioInfomation_t _AudStatus;
    HDMI_HDR_METADATA _rHdrMetadata;
    E_HDMI_DOLBY_HDR_STATE _rDolbyHdrState;
} HdmiRxObject_t;

typedef struct HDMI_PORT_INFO_T
{
    UINT8 u1UIPortId;           ///< HDMI UI Port ID, Port ID should start from 1 which corresponds to HDMI "port 1".
    UINT8 u1DriverPortId;       ///< HDMI  Driver Port ID
    BOOL bEnable;               ///<HDMI Driver Port Enable
    BOOL bCECSupport;           ///<CEC Support
    BOOL bARCSupport;           ///<ARC Support
} HDMI_PORT_INFO_T;

typedef struct HDMI_PORTS_INFO_T
{
    UINT8 u1PortNum;           ///< the number of HDMI ports in the system
    HDMI_PORT_INFO_T rHdmiPortInfo[HDMI_SWITCH_MAX-1];
} HDMI_PORTS_INFO_T;

typedef struct _HDMI_AUD_HANDLER_CTX_T
{
    BOOL fgHdmiAudInit;
    E_HDMI_SWITCH_NUM eActivePort;
    UINT8 u1ACR_DPLL_LOCK;
    UINT8 u1AudFifoFault;
    UINT8 u1AudPacket;
    UINT8 bHDMIAudFIFOflag;
    UINT8 bHDMIAudFreq;
    BOOL fgMuteAudModule;
    BOOL fgAudToggleFs;
    UINT8 u1AudToggleFsCounter;
    BOOL fgDsdOn;
    BOOL fgHbrOn;
}HDMI_AUD_HANDLER_CTX_T;

typedef struct _HDMI_DOLBY_EDR_METADATA_T
{
    UINT8 u1DmMetadataId;
    UINT8 au1MdBodyPKT[HDMI_DOLBY_EDR_MAX_BODY_LENGTH];
    UINT16 u2Length;
}HDMI_DOLBY_EDR_METADATA_T;

#ifdef CC_SUPPORT_DOLBY_VISION
typedef struct _HDMI_DOLBY_EDR_CTX_T
{
    HDMI_DOLBY_EDR_METADATA_T rHdmiDolbyEdrMetadata;
    HDMI_DOLBY_EDR_METADATA_T *prDolbyEdrMetadataBuffer;
    UINT16 u2CurrDolbyEdrMetadataIndex;
    UINT32 u4NoMetadataCount;
    BOOL fgDolbyHdrExist;
    BOOL fgFreezeVideo;
    UINT8 u1NoFreezeCounter;
    UINT8 u1UnfreezeCount;
    HAL_TIME_T tDolbyHdrMetadataReceiveTime;
}HDMI_DOLBY_EDR_CTX_T;
#endif //CC_SUPPORT_DOLBY_VISION

typedef void (*DRIVER_HDMI_ACP_CHG_CB)(UINT8 u1HdmiAcp,void* pvTag);
typedef struct _HDMI_ACP_HANDLER_CTX_T
{
    DRIVER_HDMI_ACP_CHG_CB pfAcpChgCb;
    void* pvAcpChgTag;
    UINT8 u1LastAcpType;
}HDMI_ACP_HANDLER_CTX_T;

extern E_HDMI_SWITCH_NUM eHDMICurrSwitch;
/**
 * @brief        Check HDMI port support 6G or not
 * @param        ePort---- HDMI port number
 * @retval       TRUE for support 6G, FALSE for not support
*/
extern BOOL bHDMI2IsSupport6G(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check HDMI port is valid or not
 * @param        ePort---- HDMI port number
 * @retval       TRUE for valid port, FALSE for invalid
*/
extern BOOL fgHDMI2IsValidPort(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get MHL port id
 * @retval       MHL port id
*/
extern UINT8 u1HDMI2GetMhlPortId(void);
/**
 * @brief        Get HDMI digital base address
 * @param        ePort---- HDMI port number
 * @retval       Base address
*/
extern UINT16 u2HDMI2GetDigBaseAddr(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get HDMI analog base address
 * @param        ePort---- HDMI port number
 * @retval       Base address
*/
extern UINT16 u2HDMI2GetAnaBaseAddr(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get 5V status
 * @param        ePort---- HDMI port number
 * @retval       1 for 5V exist, 0 for not exist
*/
extern UINT8 u1HDMI2_Get5VStatus(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Force 5V on for test which cannot have 5V input
 * @param        ePort---- HDMI port number
 * @param        fgEnable---- TRUE for force 5V on, FALSE for auto detect 5V
*/
extern void vHDMI2Force5vOn(E_HDMI_SWITCH_NUM ePort,BOOL fgEnable);
/**
 * @brief        Select port to be active
 * @param        ePort---- Active HDMI port number
 * @param        u1Type---- Video path number
*/
extern void vHDMI2PortSel(E_HDMI_SWITCH_NUM ePort,UINT8 u1Type);
/**
 * @brief        Select pll to active port
 * @param        ePort---- Active HDMI port number
 * @param        u1Type---- Video path number
*/
extern void vHDMI2PllSel(E_HDMI_SWITCH_NUM ePort,UINT8 u1Type);
/**
 * @brief        Set HDMI state machine
 * @param        ePort---- HDMI port number
 * @param        u1HdmiSta---- HDMI state
*/
extern void vHDMI2SetStateMachine(E_HDMI_SWITCH_NUM ePort,UINT8 u1HdmiSta);
/**
 * @brief        Get HDMI state machine
 * @param        ePort---- HDMI port number
 * @retval       HDMI state
*/
extern UINT8 bHDMI2GetStateMachine(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Stop HDMI loop
 * @param        ePort---- HDMI port number
*/
extern void vHDMI2StopFw(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Resume HDMI loop
 * @param        ePort---- HDMI port number
*/
extern void vHDMI2ResumeFw(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check HDMI is unplug or not
 * @param        ePort---- HDMI port number
 * @retval       1 for HDMI is unplug, 0 for not
*/
extern UINT8 u1HDMI2IsUnplug(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Clear HDMI unplug flag
 * @param        ePort---- HDMI port number
*/
extern void vHDMI2ClrUnplug(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HDMI unplug flag
 * @param        ePort---- HDMI port number
*/
extern void vHDMI2SetUnplug(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check if now is HPD low period
 * @param        ePort---- HDMI port number
 * @retval       1 for now is HPD low period, 0 for not
*/
extern UINT8 u1HDMI2IsHpdPulse(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HPD high period flag
 * @param        ePort---- HDMI port number
 * @param        u1Val---- 1 for HPD high, 0 for HPD low
*/
extern void vHDMI2SetHpdPulse(E_HDMI_SWITCH_NUM ePort,UINT8 u1Val);
/**
 * @brief        Check if SCDT lose or not
 * @param        ePort---- HDMI port number
 * @retval       1 for SCDT lose, 0 for SCDT exist
*/
extern UINT8 u1HDMI2IsSyncLose(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set SCDT lose flag
 * @param        ePort---- HDMI port number
 * @param        u1Val---- 1 for SCDT lose, 0 for SCDT exist
*/
extern void vHDMI2SetSyncLose(E_HDMI_SWITCH_NUM ePort,UINT8 u1Val);
/**
 * @brief        Get real time signal exist(CKDT and SCDT both exist) or not
 * @param        ePort---- HDMI port number
 * @retval       TRUE for signal exist, FALSE for not
*/
extern BOOL fgHDMI2SignalPresent(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get 420 mode
 * @param        ePort---- HDMI port number
 * @retval       TRUE for 420 mode, FALSE for not
*/
extern BOOL fgHDMI2Get420Mode(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check pixel repeat or not
 * @param        ePort---- HDMI port number
 * @retval       TRUE for pixel repeat, FALSE for not
*/
extern BOOL fgHDMI2IsPixelRepeate(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check RGB or YCbCr
 * @param        ePort---- HDMI port number
 * @retval       1 for RGB, 0 for YCbCr
*/
extern UINT8 bHDMI2InputType(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check ITU601 or ITU709
 * @param        ePort---- HDMI port number
 * @retval       1 for ITU601, 0 for ITU709
*/
extern UINT8 bHDMI2Clocrimetry(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get HDMI range mode
 * @retval       00: Default, depend on video format.
                           PC timing: full range.
                           Video timing: limited range.
                     01: limited range.
                     10: full range.
                     11: Reserved.
*/
extern UINT8 bGetHDMIRangeMode(void);
/**
 * @brief        Set HDMI range mode
 * @param        bMode---- 00: Default, depend on video format.
                                           PC timing: full range.
                                           Video timing: limited range.
                                      01: limited range.
                                      10: full range.
                                      11: Reserved.
*/
extern void vSetHDMIRangeMode(UINT8 bMode);
/**
 * @brief        Get HDMI RGB range
 * @param        ePort---- HDMI port number
 * @retval       HDMI RGB range
*/
extern UINT8 bHDMI2RgbRange(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get HDMI dule mode
 * @param        ePort---- HDMI port number
 * @retval       TRUE for HDMI dule mode, FALSE for not
*/
extern BOOL fgHDMI2GetDuleMode(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HDMI analog circuit control
 * @param        ePort---- HDMI port number
 * @param        bOnOff---- TRUE for HDMI analog circuit on, FALSE for off
*/
extern void vHDMI2EQCIRCUITCTRL(E_HDMI_SWITCH_NUM ePort,BOOL bOnOff);
/**
 * @brief        Turn HDMI termination on or off. The HDMI analog circuit will also be turned off when turn off termination here.
 * @param        ePort---- HDMI port number
 * @param        bOnOff---- TRUE for turn on termination, FALSE for turn off
*/
extern void vHDMI2TMDSCTRL(E_HDMI_SWITCH_NUM ePort,BOOL bOnOff);
/**
 * @brief        Set HDCP ddc bus control
 * @param        ePort---- HDMI port number
 * @param        bOnOff---- TRUE for turn on, FALSE for turn off
*/
extern void vHDMI2DDCBusCTRL(E_HDMI_SWITCH_NUM ePort,BOOL bOnOff);
/**
 * @brief        Get HDMI deep color status
 * @param        ePort---- HDMI port number
 * @retval       Color depth, see Enum HdmiRxBitDepth_t
*/
extern HdmiRxBitDepth_t bHDMI2DeepColorStatus(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HDMI video mute.
 * @param        ePort---- HDMI port number
 * @param        u1En---- TRUE for mute, FALSE for unmute
*/
extern void vHDMI2VideoMute(E_HDMI_SWITCH_NUM ePort,UINT8 u1En);
/**
 * @brief        Get audio channel number
 * @param        ePort---- HDMI port number
 * @retval       Audio channel number
*/
extern UINT8 bHDMI2GetAudChannelNum(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Not implemented
 * @param        fgEnable----
 * @retval
*/
extern void vHDMISpdifOutEnable(BOOL fgEnable);
/**
 * @brief        Get audio bit width
 * @param        ePort---- HDMI port number
 * @retval       Audio bit width
*/
extern UINT8 bHDMI2AudBitWidth(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get audio channel status
 * @param        ePort---- HDMI port number
 * @param        pfgCopyright:---- Pointer to copy right info
 * @param        pu1CategoryCode:---- Pointer to category code info
 * @param        p1WordLength:---- Pointer to word length info
 * @param        pfgIsPcm:---- Pointer to pcm info
*/
extern void vHDMI2AudChannelStatus(E_HDMI_SWITCH_NUM ePort, BOOL *pfgCopyright, UINT8 *pu1CategoryCode,
                           UINT8 *p1WordLength, BOOL *pfgIsPcm);
/**
 * @brief        Get audio sample frequency
 * @retval       Audio sample frequency
*/
extern UINT8 u1HDMI2GetAudioSamplingFreq(void);
/**
 * @brief        Get audio status
 * @retval       1 for audio NG, 0 for audio OK
*/
extern UINT8 bHDMI2AudStatus(void);
/**
 * @brief        Check dsd audio exist
 * @retval       TRUE for dsd audio exist, FALSE for dsd audio not exist
*/
extern BOOL fgHDMI2AudDsdOn(void);
/**
 * @brief        Check hbr audio exist
 * @retval       TRUE for hbr audio exist, FALSE for hbr audio not exist
*/
extern BOOL fgHDMI2AudHbrOn(void);
/**
 * @brief        Check linear PCM audio
 * @param        ePort---- HDMI port number
 * @retval       TRUE for linear PCM audio, FALSE for not
*/
extern BOOL fgHDMI2IsPCM(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        value1----
 * @param        value2----
 * @param        bSDINOVR----
*/
extern void vHDMI2AudioSigmaDeltaSetting(UINT32 value1, UINT8 value2, UINT8 bSDINOVR);
/**
 * @brief        Set HDMI APLL2 control parameter
 * @param        value1----
 * @param        value2----
 * @param        target----
 * @param        bSDINOVR----
*/
extern void vHDMI2AudioSigmaDeltaFineTuneSetting(UINT32 value1, UINT8 value2, UINT32 target, UINT8 bSDINOVR);
/**
 * @brief        Get audio PCW mode
 * @retval       TRUE for HW mode, FALSE for SW mode
*/
extern BOOL fgHDMI2AudioPCWMode(void);
#ifdef CC_SUPPORT_DOLBY_VISION
/**
 * @brief        Check dolby HDR exist
 * @param        u1Path---- Video path number
 * @retval       TRUE dolby HDR exist, FALSE for not exist
*/
extern E_HDMI_DOLBY_HDR_STATE eHDMI2DolbyHdrState(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Check dolby HDR exist
 * @param        u1Path---- Video path number
 * @retval       TRUE dolby HDR exist, FALSE for not exist
*/
extern BOOL fgHDMI2DolbyHDRExist(UINT8 u1Path);
/**
 * @brief        Init dolby EDR metadata buffer
*/
extern void vHDMI2InitDolbyEdrMetadataBuffer(void);
/**
 * @brief        Get dolby EDR metadata buffer
 * @param        u2Index---- Metadata index
 * @param        *prEdrMetadata---- Pointer to metadata structure HDMI_DOLBY_EDR_METADATA_T
 * @retval       TRUE for valid metadata, FALSE no valid metadata
*/
extern BOOL fgHDMI2GetDolbyEdrMetadataBuff(UINT16 u2Index, HDMI_DOLBY_EDR_METADATA_T *prEdrMetadata);
/**
 * @brief        Freeze/Unfreeze video for Dolby video error handling.
 * @param        u1Path---- Video path number
 * @param        fgFreeze---- TRUE for freeze, FALSE for unfreeze
 * @param        fgForce---- TRUE for ignore signal status, FALSE require stable signal status in DVI.
*/
extern void vHDMI2DolbyEdrFreezeVideo(UINT8 u1Path,BOOL fgFreeze, BOOL fgForce);
#endif //CC_SUPPORT_DOLBY_VISION
/**
 * @brief        Init HDMI module
*/
extern void vHDMI2Init(void);
/**
 * @brief        Get last recorded input TMDS clock
 * @param        ePort---- HDMI port number
 * @retval       TMDS clock in Khz
*/
extern UINT32 u4HDMI2GetTmdsClkKhz(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get current input TMDS clock
 * @param        ePort---- HDMI port number
 * @retval       TMDS clock in Khz
*/
extern UINT32 u4HDMI2GetCurrentTmdsClkKhz(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get clock rate enum
 * @param        u4ClkKhz---- Clock rate in Khz
 * @retval       Clock rate enum
*/
extern HdmiTmdsClkRate_t bHDMI2GetRefClkRate(UINT32 u4ClkKhz);
/**
 * @brief        Get clock rate enum
 * @param        u4ClkKhz---- Clock rate in Khz
 * @retval       Clock rate enum
*/
extern MhlTmdsClkRate_t bMHL2GetRefClkRate(UINT32 u4ClkKhz);
/**
 * @brief        Check hdcp stable
 * @param        ePort---- HDMI port number
 * @retval       1 for HDCP stable, 0 for unstable
*/
extern UINT8 u1IsHDMI2HdcpStable(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check hdcp authentication done
 * @param        ePort---- HDMI port number
 * @retval       TRUE for authentication done, FALSE for not
*/
extern BOOL fgIsHDMI2HdcpAuthDone(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check hdcp2x decrypting on
 * @param        ePort---- HDMI port number
 * @retval       TRUE for hdcp2x decrypting on, FALSE for hdcp2x decrypting off
*/
extern BOOL fgIsHDMI2HDCP2xExist(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Set HDMI fast switch flag
 * @param        ePort---- HDMI port number
 * @param        u1Val---- 1 for fast switch on, 0 for fast switch off
*/
extern void vSetHdmi2FastSw(E_HDMI_SWITCH_NUM ePort, UINT8 u1Val);
/**
 * @brief        Check HDMI fast switch flag
 * @param        ePort---- HDMI port number
 * @retval       1 for fast switch on, 0 for fast switch off
*/
extern UINT8 u1IsHdmi2FastSw(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check HDMI resolution stable
 * @param        ePort---- HDMI port number
 * @retval       1 for HDMI resolution stable, 0 for unstable
*/
extern UINT8 u1IsHDMI2Stable(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Check HDMI/DVI mode
 * @param        eHdmiPort---- HDMI port number
 * @retval       See Enum HdmiRxMode_t
*/
extern UINT8 bHDMI2GetHdmiMode(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI color space
 * @param        eHdmiPort---- HDMI port number
 * @retval       See Enum HdmiRxClrSpc_t
*/
extern HdmiRxClrSpc_t eHDMI2GetHdmiClrSpc(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI scan info
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI scan info
*/
extern UINT8 bHDMI2ScanInfo(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI aspect ratio
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI aspect ratio
*/
extern UINT8 bHDMI2AspectRatio(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI AFD
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI AFD
*/
extern UINT8 bHDMI2AFD(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Check 422 input
 * @param        eHdmiPort---- HDMI port number
 * @retval       1 for 422 input, 0 for not
*/
extern UINT8 bHDMI2422Input(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Transfer color space to YC422
 * @param        eHdmiPort---- HDMI port number
 * @param        fgEnable---- TRUE for enable, FALSE for disable
*/
extern void vHDMI2TransferToVDP422(E_HDMI_SWITCH_NUM eHdmiPort,BOOL fgEnable);
/**
 * @brief        Get HDMI ITC flag
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI ITC flag
*/
extern UINT8 bHDMI2ITCFlag(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI ITC content
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI ITC content
*/
extern UINT8 bHDMI2ITCContent(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get hdcp1x status
 * @param        eHdmiPort---- HDMI port number
 * @retval       See HdmiRxHdcp1Status_t for bit map
*/
extern UINT8 bHDMIHDCP1StatusGet(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get hdcp2x auth status
 * @param        eHdmiPort---- HDMI port number
 * @retval       Enum for auth status, see HDMI_RX_HDCP2X_AUTH_STATUS_T
*/
extern HDMI_RX_HDCP2X_AUTH_STATUS_T eHDMI2GetHdcp2xAuthStatus(E_HDMI_SWITCH_NUM ePort);
/**
 * @brief        Get HDMI infoframe for hdmi_if interface
 * @param        eHdmiPort---- HDMI port number
 * @param        *ptInfo---- Pointer to HdmiRxInfo_t
 * @param        u1Type---- Infoframe type, see HdmiInfoFrameID_t
 * @retval
*/
extern UINT8 bHDMI2GetInfoFrameToIF(E_HDMI_SWITCH_NUM eHdmiPort,HdmiRxInfo_t *ptInfo,UINT8 u1Type);
/**
 * @brief        Get HDMI SPD UP status
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI SPD UP status
*/
extern UINT8 u1HDMI2GetUPStatus(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Check HDMI SPD UP status change
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI SPD UP status change type
*/
extern UINT8 u1HDMI2GetUPStaChg(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI SPD 4K ps status
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI SPD 4K ps status
*/
extern UINT8 u1HDMI2Get4KPStatus(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Set ACP change callback function
 * @param        pfAcpChgCb---- Pointer to DRIVER_HDMI_ACP_CHG_CB
 * @param        *tag---- Externed tag
*/
extern void vHDMI2SetAcpChgCallback(DRIVER_HDMI_ACP_CHG_CB pfAcpChgCb,void *tag);
/**
 * @brief        Get HDMI HDR EOTF value
 * @param        eHdmiPort---- HDMI port number
 * @retval       HDMI HDR EOTF value
*/
extern UINT8 u1HDMI2GetHdrEOTF(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Get HDMI HDR metadata
 * @param        eHdmiPort---- HDMI port number
 * @param        *prHdrMetadata---- Pointer to HDMI_HDR_METADATA
*/
extern void vHDMI2GetHdrMetaData(E_HDMI_SWITCH_NUM eHdmiPort,HDMI_HDR_METADATA *prHdrMetadata);
/**
 * @brief        Check HDCP key valid
 * @param        eHdcpKeyCheckMode---- Check key type, see HDMI_HDCP_KEY_CHECK_MODE_T
 * @retval       1 for key valid, 0 for key invalid
*/
extern UINT8 u1HDMIHDCPKeyCheckValid(HDMI_HDCP_KEY_CHECK_MODE_T eHdcpKeyCheckMode);
#ifdef __MODEL_slt__
/**
 * @brief        Sample slt golden CRC
 * @param        eHdmiPort---- HDMI port number
 * @retval       TRUE for sucess, FALSE for fail
*/
extern BOOL bHDMI2SltCRCSample(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Check slt CRC same as golden CRC
 * @param        eHdmiPort---- HDMI port number
 * @retval       TRUE for same as golden CRC, FALSE for not
*/
extern BOOL bHDMI2SltCRCCheck(E_HDMI_SWITCH_NUM eHdmiPort);
#endif //__MODEL_slt__
/**
 * @brief        Print HDCP1.4 status
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2PrintHdcp14Status(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Print HDCP2.2 status
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2PrintHdcp22Status(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Print HDMI general status
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2PrintHdmiGeneralStatus(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Print HDMI video status
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2PrintHdmiVideoStatus(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Print HDMI audio status
 * @param        eHdmiPort---- HDMI port number
*/
extern void vHDMI2PrintHdmiAudioStatus(E_HDMI_SWITCH_NUM eHdmiPort);
/**
 * @brief        Print HDMI status
 * @param        eHdmiPort---- HDMI port number
 * @retval
*/
extern void vHDMI2PrintHdmiStatus(E_HDMI_SWITCH_NUM eHdmiPort);
#ifdef CC_SUPPORT_STR
/**
 * @brief        HDMI fastboot suspend function
*/
extern void HDMI2_pm_suspend(void);
/**
 * @brief        HDMI fastboot resume function
*/
extern void HDMI2_pm_resume(void);
#endif //CC_SUPPORT_STR
/**
 * @brief        HDMI main loop handler
*/
extern void vHDMI2MainLoop(void);

//Customized functions defined in hdmi_conf.c >>> BEGIN
extern void vHDMIHPDHigh(E_HDMI_SWITCH_NUM eSwitch);
extern void vHDMIHPDLow(E_HDMI_SWITCH_NUM eSwitch);
extern UINT8 bHDMIPort5VStatus(E_HDMI_SWITCH_NUM ePort);
extern void vHDMISetSwitch(E_HDMI_SWITCH_NUM eSwitch,UINT8 u1Type);
//Customized functions defined in hdmi_conf.c >>> END
//Customized functions defined in mtk_hdmi.h >>> BEGIN
extern void vHDMIPortInfoInit(void);
extern void vHDMIGetPortsInfo(HDMI_PORTS_INFO_T* prPortsInfo);
extern E_HDMI_SWITCH_NUM eHDMIGetPhysicalPortId(E_HDMI_SWITCH_NUM eVirPort);
extern E_HDMI_SWITCH_NUM eHDMIGetVirtualPortId(E_HDMI_SWITCH_NUM ePhyPort);
#ifdef CC_HDMI_POH_SUPPORT
extern void vHDMIPohConfigInit(void);
extern BOOL fgHDMIPohEnabled(E_HDMI_SWITCH_NUM ePort);
extern void vHDMIGetPohConfig(E_HDMI_SWITCH_NUM ePort,HDMI_POH_CONFIG_T *prPohConfig);
#endif
//Customized functions defined in mtk_hdmi.h >>> END
extern void vHDMI2SetLateResumeFlag(BOOL fgLateResume);
extern UINT8 u1HDMI2GetDolbyVision(HdmiRxPort eHdmiPort);
#endif

