/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------

//============================================================================
// NOTE:
//      In this file, all function cannot call sub-function!!!
//============================================================================
#include "rs232.h"

#include "x_ckgen.h"
#include "x_assert.h"
#include "x_timer.h"
#include "x_bim.h"
#include "x_pdwnc.h"
#include "drvcust_if.h"
#include "c_model.h"
#include "hw_pll_group.h"
#include "hw_hpcpll.h"
#include "hw_ifpll.h"
#include "hw_pdwnc.h"
//#include "hw_ifpga.h"
#include "hw_ospe.h"
//#include "hw_mempll.h"
#include "hw_tg.h"
#include "x_hal_5381.h"
#include "chip_txt.h"
#include "hw_gpu_mfg_top.h"

extern UINT32 IS_SupportCPUHiSpeed(VOID);

void CHIP_ClockSetting(void);
void CHIP_BimInit(void);
void CHIP_1stBootRegInit(void);

void CHIP_Delay_us(UINT32 u4Micros)
{
#ifdef CC_FPGA
    volatile UINT32 i;

    for (i = 0; i < (24 * u4Micros) * 100; i++)
    {
        ;
    }
#else
    UINT32 u4Xtal = 24 * u4Micros;
    UINT32 u4Org = BIM_READ32(REG_RW_TIMER2_LOW);
    ASSERT(u4Micros < 0xAAAAAAA);   //check 32-bit
    while (u4Org - BIM_READ32(REG_RW_TIMER2_LOW) < u4Xtal);
#endif
}

void CHIP_ClockSetting(void)
{
    int reg_val = 0;
	unsigned int efuse_reg_val = 0;

    // 0. Select CPU/BUS/SRAM/BIST to Xtal before PLL configuration
    vIO32Write4B(IO_VIRT + REG_MP2_PLL_DIVIDER_CFG, 0x0FF); // Select CPU as XTAL
    vIO32WriteFldAlign(CKGEN_CPU_CKCFG, 0, FLD_BUS_CK_SEL); // CPU_CK_SEL is removed
    vIO32WriteFldAlign(CKGEN_REG_AXI_CKCFG, 0, FLD_AXI_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_ABIST_SRAM_CKCFG, 0x0, FLD_ABIST_SRAM_SEL);
    vIO32WriteFldAlign(CKGEN_REG_BIST_CKCFG, 0x0, FLD_BIST_SEL);
    CHIP_Delay_us(40);

    // Xtal Gm control, increase driving current to 12X
    vIO32WriteFldMulti(PDWNC_XTALCFG0, P_Fld(0xF, FLD_RG_XTAL_GM) |
                                       P_Fld(0x0, FLD_RG_XTAL_AAC_EN));

    // 1. Init SYSPLL, G3DPLL, VDECPLL, DMUXPLL, HPCPLL
    // DA_PLL_SDM_PWR_ON = 1
    vIO32WriteFldMulti(CKGEN_ANA_PLLGP_CFG1, P_Fld(1, FLD_DA_DMUXPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_VDECPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_VDECSOCPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_G3DPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_SYSPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_HPCPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_VGAPLL_SDM_PWR_ON) |
                                             P_Fld(1, FLD_DA_ARMPLL_SDM_PWR_ON));
    // DA_PLL_SDM_ISO_EN = 0
    vIO32WriteFldMulti(CKGEN_ANA_PLLGP_CFG1, P_Fld(0, FLD_DA_DMUXPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_VDECPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_VDECSOCPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_G3DPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_SYSPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_HPCPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_VGAPLL_SDM_ISO_EN) |
                                             P_Fld(0, FLD_DA_ARMPLL_SDM_ISO_EN));
    CHIP_Delay_us(50);

    // Power down PLL
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG0, 0x0, FLD_RG_SYSPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG1, 0x0, FLD_RG_VDECPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG2, 0x0, FLD_RG_VDECSOCPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG3, 0x0, FLD_RG_G3DPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG4, 0x0, FLD_RG_DMUXPLL_EN);
    vIO32WriteFldAlign(REG_HPCPLL_CFG0, 0x0, FLD_RG_HPCPLL_EN);
    vIO32WriteFldAlign(REG_ARMPLL_CFG0, 0x0, FLD_RG_ARMPLL_EN);

    // Set PLLGP_BIAS_EN = 0; disable if ROM code is enabled.
    //vIO32WriteFldAlign(REG_PLL_GROUP_CFG11,  0x0,  FLD_RG_PLLGP_BIAS_EN);

    // Set SYSPLL for 648 MHz
    // SYSPLL_SDM_FRA_EN = 0 for integer application
    // SYSPLL = (Xtal * SYSPLL_SDM_PCW) / SYSPLL_PREDIV / SYSPLL_POSDIV
    // SYSPLL = (Xtal * 108) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG0, P_Fld(0x00, FLD_RG_SYSPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_SYSPLL_PREDIV) |
                                           P_Fld(0x00, FLD_RG_SYSPLL_POSDIV) |
                                           P_Fld(0x6C, FLD_RG_SYSPLL_SDM_PCW));

    // Set VDECPLL for 552 MHz
    // VDECPLL_SDM_FRA_EN = 0 for integer application
    // VDECPLL = (Xtal * VDECPLL_SDM_PCW) / VDECPLL_PREDIV / VDECPLL_POSDIV
    // VDECPLL = (Xtal * 92) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG1, P_Fld(0x00, FLD_RG_VDECPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_VDECPLL_PREDIV) |
                                           P_Fld(0x02, FLD_RG_VDECPLL_POSDIV) |
                                           P_Fld(0x5C, FLD_RG_VDECPLL_SDM_PCW));

    // Set VDECSOCPLL for 456 MHz
    // VDECSOCPLL_SDM_FRA_EN = 0 for integer application
    // VDECSOCPLL = (Xtal * VDECSOCPLL_SDM_PCW) / VDECSOCPLL_PREDIV / VDECSOCPLL_POSDIV
    // VDECSOCPLL = (Xtal * 76) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG2, P_Fld(0x00, FLD_RG_VDECSOCPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_VDECSOCPLL_PREDIV) |
                                           P_Fld(0x02, FLD_RG_VDECSOCPLL_POSDIV) |
                                           P_Fld(0x4C, FLD_RG_VDECSOCPLL_SDM_PCW));

    // Set G3DPLL for 528 MHz
    // G3DPLL_SDM_FRA_EN = 0 for integer application
    // G3DCPLL = (Xtal * G3DPLL_SDM_PCW) / G3DPLL_PREDIV / G3DPLL_POSDIV
    // G3DCPLL = (Xtal * 88) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG3, P_Fld(0x00, FLD_RG_G3DPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_G3DPLL_PREDIV) |
                                           P_Fld(0x02, FLD_RG_G3DPLL_POSDIV) |
                                           P_Fld(0x58, FLD_RG_G3DPLL_SDM_PCW));

    // Set DMUXPLL for 384 MHz
    // DMUXPLL_SDM_FRA_EN = 0 for integer application
    // DMUXPLL = (Xtal * DMUXPLL_SDM_PCW) / DMUXPLL_PREDIV / DMUXPLL_POSDIV
    // DMUXPLL = (Xtal * 64) / 1 / 4
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG4, P_Fld(0x00, FLD_RG_DMUXPLL_SDM_FRA_EN) |
                                           P_Fld(0x00, FLD_RG_DMUXPLL_PREDIV) |
                                           P_Fld(0x02, FLD_RG_DMUXPLL_POSDIV) |
                                           P_Fld(0x01, FLD_RG_DMUXPLL_RESERVE) | // for Xtal = 24Mhz base
                                           P_Fld(0x3F, FLD_RG_DMUXPLL_SDM_PCW));//195MHZ--0x41

    // Set HPCPLL as 1032 MHz
    // HPCPLL_SDM_FRA_EN = 0 for integer application
    // HPCPLL = (Xtal * HPCPLL_SDM_PCW) / HPCPLL_PREDIV / HPCPLL_POSDIV
    // HPCPLL = (Xtal * 86) / 1 / 2
    vIO32WriteFldMulti(REG_HPCPLL_CFG0, P_Fld(0x00, FLD_RG_HPCPLL_SDM_FRA_EN) |
                                        P_Fld(0x00, FLD_RG_HPCPLL_PREDIV) |
                                        P_Fld(0x01, FLD_RG_HPCPLL_POSDIV));
    {
        UINT32 u4HPCPLL_SDM_PCW = 0x56;

        #ifdef CC_CPU_SPEED_TEST_ENABLE
        if ((u4IO32Read4B(0x10028128) == 0) && u4IO32Read4B(0x10028100) == 0)
        {
            vIO32Write4B(0x10008150, 0xFFFFFFFF);
            vIO32Write4B(0x10008154, u4HPCPLL_SDM_PCW);
        }
        else
        {
            // Get HPCPLL setting
            if (u4IO32Read4B(0x10008150) == 0xFFFFFFFF)
            {
                u4HPCPLL_SDM_PCW = u4IO32Read4B(0x10008154);
            }
        }
        #endif // CC_CPU_SPEED_TEST_ENABLE
        vIO32WriteFldAlign(CKGEN_ANA_PLLGP_CFG0, u4HPCPLL_SDM_PCW, FLD_ANA_HPCPLL_SDM_PCW_INTEGER);
    }

    // Set ARMPLL for 888Mhz
    // ARMPLL_SDM_FRA_EN = 0 for integer application
    // ARMPLL = (Xtal * ARMPLL_SDM_PCW) / ARMPLL_PREDIV / ARMPLL_POSDIV
    // ARMPLL = (Xtal * 74) / 1 / 2
    vIO32WriteFldMulti(REG_ARMPLL_CFG0, P_Fld(0x00, FLD_RG_ARMPLL_SDM_FRA_EN) |
                                        P_Fld(0x00, FLD_RG_ARMPLL_PREDIV) |
                                        P_Fld(0x01, FLD_RG_ARMPLL_POSDIV));
    vIO32WriteFldAlign(REG_ARMPLL_CFG1, 0x4A, FLD_RG_ARMPLL_SDM_PCW_INTEGER);

    // 2. Power on PLL
    // Set PLLGP_BIAS_EN = 1
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG11, 0x1, FLD_RG_PLLGP_BIAS_EN);
    CHIP_Delay_us(50);

    // Power on G3DPLL, SYSPLL, VDECPLL, HPCPLL, ARMPLL
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG0, 0x1, FLD_RG_SYSPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG1, 0x1, FLD_RG_VDECPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG2, 0x1, FLD_RG_VDECSOCPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG3, 0x1, FLD_RG_G3DPLL_EN);
    vIO32WriteFldAlign(REG_HPCPLL_CFG0, 0x1, FLD_RG_HPCPLL_EN);
    vIO32WriteFldAlign(REG_ARMPLL_CFG0, 0x1, FLD_RG_ARMPLL_EN);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG4, 0x1, FLD_RG_DMUXPLL_EN);

    // Enable DDS setting
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG7, 0x1, FLD_RG_HADDS1_PWDB);
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG8, 0x1, FLD_RG_XDDS_PWDB);
    vIO32WriteFldMulti(REG_PLL_GROUP_CFG10, P_Fld(1, FLD_RG_DDDS1_PWDB) |
                                            P_Fld(1, FLD_RG_DDDS2_PWDB));
    CHIP_Delay_us(50);

    // 3. Set IFPLL
    // Load BAIS current
    vIO32WriteFldAlign(REG_IFPLL_CFG1, 0x1, FLD_RG_IFPLL_BIAS_EN); // IFPLL_BIAS_EN = 1
    vIO32WriteFldAlign(REG_IFTOP_CFG1, 0x0, FLD_RG_DMTOP_BG_PWD);  // DMTOP_BG_PWD = 0
    CHIP_Delay_us(10);

    // Load PLL 1728Mhz default
    // frequency equation: 24MHz / PREDIV * (FBKDIV + 1) * FBKSEL / POSDIV
    //   PREDIV: 00:1 01:2 10:4
    //   POSDIV: 00:1 01:2 10:4
    //   FBKSEL: 00:1 01:2 10:4
    vIO32WriteFldAlign(REG_IFPLL_CFG0, 0x0, FLD_RG_IFPLL_EN); // IFPLL_EN = 0
    vIO32WriteFldMulti(REG_IFPLL_CFG0, P_Fld(0x11, FLD_RG_IFPLL_FBKDIV) |
                                       P_Fld(0x02, FLD_RG_IFPLL_FBKSEL) |
                                       P_Fld(0x02, FLD_RG_IFPLL_POSDIV) |
                                       P_Fld(0x00, FLD_RG_IFPLL_PREDIV) |
                                       P_Fld(0x0D, FLD_RG_IFPLL_IR) |
                                       P_Fld(0x00, FLD_RG_IFPLL_BR) |
                                       P_Fld(0x00, FLD_RG_IFPLL_BC) |
                                       P_Fld(0x0C, FLD_RG_IFPLL_IC) |
                                       P_Fld(0x02, FLD_RG_IFPLL_BP));
    vIO32WriteFldMulti(REG_IFPLL_CFG1, P_Fld(0x00, FLD_RG_IFPLL_DIVEN) |
                                       P_Fld(0x01, FLD_RG_IFPLL_BAND));
    vIO32WriteFldMulti(REG_IFPLL_CFG1, P_Fld(0x1, FLD_RG_IFPLL_AUTOK_EN) |
                                       P_Fld(0x0, FLD_RG_IFPLL_AUTOK_LOAD));

    // start calibration
    CHIP_Delay_us(1);

    // power on IFPLL
    vIO32WriteFldAlign(REG_IFPLL_CFG0, 1, FLD_RG_IFPLL_EN);
    CHIP_Delay_us(180);

    // Read band results from RGS_PLL_AUTOK_BAND
    // Write K band results to RG_IFPLL_BAND
    vIO32WriteFldAlign(REG_IFPLL_CFG1, IO32ReadFldAlign(RGS_IFPLL_CFG0, FLD_RGS_PLL_AUTOK_BAND), FLD_RG_IFPLL_BAND);

    // stop calibration
    vIO32WriteFldAlign(REG_IFPLL_CFG1, 0, FLD_RG_IFPLL_AUTOK_EN);
    CHIP_Delay_us(20);

    // enable IFPLL output gating to DMODADC, ckgen, CVBS, ADAC
    vIO32WriteFldAlign(REG_IFTOP_CFG2, 0x01D, FLD_RG_DMTOP_REV0); // DMTOP_REV0 = 0x0D

    // 4. Init Ethernet PLL, USB PLL
    // XPLL = Xtal(24Mhz) *  XPLL_FBDIV(25)
    // Default XPLL = 24Mhz * 25
    // pls_fbdiv_pre[15:9] Feedback divide ratio = 25
    // ethtop_dmy_pre[7:0]
    //    0: the input xtal clk shoule be 27MHZ
    //    1: the input xtal clk shoule be 24MHZ
    // Set XPLL for 600 Mhz

    reg_val = u4IO32Read4B(0x1003231C);
    vIO32Write4B(0x1003231C, ((u4IO32Read4B(0x1003231C) & 0xffff0000) | 0x00003001));

    if (IS_IC_5891_ES1()) {
        //todo
    } else {
        reg_val = u4IO32Read4B(0x1003231C) | (reg_val & (0x1 << 8));
        vIO32Write4B(0x1003231C, reg_val);
    }

    // already do this in step0
    CHIP_Delay_us(500);
    vIO32WriteFldAlign(CKGEN_REG_USB_CKCFG, 1, FLD_USB_SEL);
    vIO32WriteFldAlign(CKGEN_USB_CLK_CKCFG, 1, FLD_USBPHY_CK_DIV9);

    //keep RG_USB20_INTR_EN=0b, use external R current
    vIO32Write4B(0x10059800, (u4IO32Read4B(0x10059800) | 0x00008000)); // RG_USB20_USBPLL_FORCE_ON = 1b
    vIO32Write4B(0x10059a14, (u4IO32Read4B(0x10059a14) | 0x0000800));  // RG_USB20_HS_100U_U3_EN = 1b
    vIO32Write4B(0x10059b14, (u4IO32Read4B(0x10059b14) | 0x0000800));  // RG_USB20_HS_100U_U3_EN = 1b

    // change USB2.0 DRAM priority from group2 to group1
    vIO32Write4B(0x1000d5ac, (u4IO32Read4B(0x1000d5ac) | 0x0f000000));
    //add ssusb port 2 clock settings
    vIO32Write4B(0x1000d47c,(u4IO32Read4B(0x1000d47c) | 0x00000003));

    // 5. Select CPU clock to HPCPLL and Bus clock
    // cpu clock = HPCPLL , bus clock = 216Mhz
    /*
    Clock source selection[3:0]
        0: xtal_ck
        1: cpupll_ck
        2: enetpll_ck (600MHZ)
        3: syspll_ck (648MHz)
        4: usbpll_ck (480MHz)
        5: sawlesspll_ck (432MHz)
        6: enetpll_d2_ck (300MHZ)
        7: usbpll_d2_ck (240MHz )
        8: sawlesspll_d2_ck (216MHz )
        9: sawlesspll_d3_ck (144MHz )
        10: usbpll_d4_ck (120MHz )
        11: sawlesspll_d4_ck (108MHZ)
        12: enetpll_d4_ck (150MHZ)
        13: MEM_CK
        14: slow_ck
        15: bist_ck
    BUS_CK selection[10:8]
        0: bus_ck = xtal_ck
        1: bus_ck = sawlesspll_d2_ck
        2: bus_ck = usbpll_d2p5_ck
        3: bus_ck = syspll_d4_ck
        4: bus_ck = sawlesspll_d3_ck
        5: bus_ck = usbpll_d4_ck/dmuxpll_d2 (EPS1 later)
        6: bus_ck = syspll_d3_ck
        7: bus_ck = slow_ck
    */
    vIO32WriteFldAlign(CKGEN_CPU_CKCFG, 6, FLD_BUS_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_AXI_CKCFG, 2, FLD_AXI_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_SMI_CKCFG, 1, FLD_SMI_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_SMI_BUS_CKCFG, 1, FLD_SMI_BUS_CK_SEL);
    BIM_WRITE32(REG_MISC2,BIM_READ32(REG_MISC2)|SMI1_COMMON_CLKEN|SMI0_COMMON_CLKEN);

    // MCU_BIU clock must be running at 1/2 CPUPLL clock
    IO_WRITE32(0x0, 0x10078640, (IO_READ32(0x0, 0x10078640) & 0xFFFFFFE0) | 0x12); // MCUCFG_ACLKEN_DIV[4:0] = 0x12
    CHIP_Delay_us(1);
    vIO32Write4B(IO_VIRT + REG_MP2_PLL_DIVIDER_CFG, 0x2FF);
    BIM_WRITE32(REG_CKDIV, 0x18);

    // 6. set common clk
    // C27M = sawless / 16
    vIO32WriteFldAlign(CKGEN_REG_C27M_CKCFG, 0x1, FLD_C27M_SEL);

    // 7. setup vopll
    // vopll source xtal
    vIO32WriteFldAlign(REG_PLL_GROUP_CFG10, 2, FLD_RG_DDDS1_CKSEL);

    // vopll setting
    vIO32WriteFldAlign(REG_LVDS_TX_PLL_CFG, 0, REG_LVDS_TX_PLL_CFG_VPLL_BG_CKEN);
    vIO32Write4B(REG_LVDSTX_PLL_CFG0, 0x00000907);
    vIO32Write4B(REG_LVDSTX_PLL_CFG1, 0x94188402);
    vIO32Write4B(REG_LVDSTX_PLL_CFG2, 0x00200033);
    vIO32Write4B(REG_LVDSTX_PLL_CFG3, 0x2e840000);
    vIO32Write4B(REG_LVDSTX_PLL_CFG4, 0x83548000);

    // Power on
    CHIP_Delay_us(10);
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG1, 0, REG_LVDSTX_PLL_CFG1_VPLL_PWD);
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_LVDS_PWD);
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BG_VB1_TX_PWD);
    vIO32WriteFldAlign(REG_LVDSTX_PLL_CFG0, 0, REG_LVDSTX_PLL_CFG0_VPLL_BIAS_BYP);

    // OCLK and TCLK Common
    vIO32WriteFldAlign(CKGEN_MBIST_CK_CTRL, 0, FLD_BIST_VPCLK_SEL);

    // OCLK Path
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, FLD_OCLK_SEL_LVDS, FLD_OCLK_SEL);
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0, FLD_OCLK_TSTSEL);

    // TCLK Path
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 1, FLD_TCLK_SEL);
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 0, FLD_TCLK_TSTSEL);

    // VB1 75MHz
    vIO32WriteFldAlign(CKGEN_TCLK_TEST, 2, FLD_CLK75M2_CK_SEL);
    vIO32WriteFldAlign(CKGEN_OCLK_TEST, 2, FLD_VB1_CK_SEL);

    // VB1 reset
    vIO32WriteFldAlign(TMGR0, 0, TMGR0_CKRST_CFG_VB1);
    vIO32WriteFldAlign(TMGR2, 0, TMGR2_CKRST_CFG_VB1OSD);
    CHIP_Delay_us(10);
    vIO32WriteFldAlign(TMGR0, 1, TMGR0_CKRST_CFG_VB1);
    vIO32WriteFldAlign(TMGR2, 1, TMGR2_CKRST_CFG_VB1OSD);

    // 8. setup serial flash pll
    //   0: 3.375 MHz   1: 5.4 MHz      2: 6.75 MHz     3: 9 MHz
    //   4: 13.5 MHz    5: 27 MHz       6: 30 MHz       7: 48 MHz
    //   8: 54 MHz      9: 72 MHz       10: 108 MHz     11: 0.84375 MHz
    IO_WRITE32(0, 0x100087B0, 5);

    // 9. setup sram/BIST clock
    //   0: xtal_d1_ck (24MHz)
    //   1: usb_phy_pll_d2_ck (240MHz)
    //   2: sawlesspll_d2_ck (216MHz)
    //   3: usb_phy_pll_d3_ck (160MHz)
    vIO32WriteFldAlign(CKGEN_REG_ABIST_SRAM_CKCFG, 0x2, FLD_ABIST_SRAM_SEL);

#if 0 // only for ROM code
    // Speed up the ROM code clock
    // BIST clock selection
    // 0 : xtal (24Mhz)
    // 1 : sawlesspll (432Mhz)
    // 2 : SYSPLL / 2 (324Mhz)
    // 3 : sawlesspll / 3 (144Mhz)
    vIO32WriteFldAlign(CKGEN_REG_BIST_CKCFG, 0x2, FLD_BIST_SEL);
#endif

    // 10. setup nand pll (0: XTAL 24Mhz, 6: 192Mhz), and improve nand PIN driving
    vIO32WriteFldAlign(CKGEN_REG_NFI_CKCFG, 0x6, FLD_NFI_SEL);

    // 11. init USB3
    #if defined(CC_MT5893)
    vIO32WriteFldAlign(CKGEN_REG_XHCI_CKCFG, 0x1, FLD_XHCI_CK_SEL); //108MHz
    #else
    vIO32WriteFldAlign(CKGEN_BLOCK_CKEN_CFG1, 0x1, FLD_SSUSB_TOP_CKEN);
    #endif
#if 1 //tmp setting brfore EFUSE imp

		efuse_reg_val = u4IO32Read4B(0x100086d8) & 0xffffffff; //efuse calibration value
		if (efuse_reg_val == 0)
		{
			reg_val = u4IO32Read4B(0x100D0B34); //SSUSB_SIFSLV_U3PHYA_BASE
			reg_val &= ~0x3f;
			reg_val |= 0x2C; //RG_SSUSB_IEXT_INTR_CTRL
			vIO32Write4B(0x100D0B34, reg_val);
	
			reg_val = u4IO32Read4B(0x100D0910); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val |= 0x90000000; //rg_ssusb_tx_impsel
			vIO32Write4B(0x100D0910, reg_val);
	
			reg_val = u4IO32Read4B(0x100D0914); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val |= 0x90000000; //rg_ssusb_rx_impsel
			vIO32Write4B(0x100D0914, reg_val);

			reg_val = u4IO32Read4B(0x100C0B34); //SSUSB_SIFSLV_U3PHYA_BASE
			reg_val &= ~0x3f;
			reg_val |= 0x2C; //RG_SSUSB_IEXT_INTR_CTRL
			vIO32Write4B(0x100C0B34, reg_val);
	
			reg_val = u4IO32Read4B(0x100C0910); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val |= 0x90000000; //rg_ssusb_tx_impsel
			vIO32Write4B(0x100C0910, reg_val);
	
			reg_val = u4IO32Read4B(0x100C0914); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val |= 0x90000000; //rg_ssusb_rx_impsel
			vIO32Write4B(0x100C0914, reg_val);
		}
		else
		{
			reg_val = u4IO32Read4B(0x100C0B34); //SSUSB_SIFSLV_U3PHYA_BASE
			reg_val &= ~(0x3f);
			reg_val = reg_val | (efuse_reg_val & (0x3f)); //RG_SSUSB_IEXT_INTR_CTRL
			vIO32Write4B(0x100C0B34, reg_val);
	
			reg_val = u4IO32Read4B(0x100C0910); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val = reg_val | 0x80000000 | ((efuse_reg_val & (0x7C0))<<18); //rg_ssusb_tx_impsel
			vIO32Write4B(0x100C0910, reg_val);
	
			reg_val = u4IO32Read4B(0x100C0914); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val = reg_val | 0x80000000 | ((efuse_reg_val & 0xf800) << 13); //rg_ssusb_rx_impsel
			vIO32Write4B(0x100C0914, reg_val);

			reg_val = u4IO32Read4B(0x100D0B34); //SSUSB_SIFSLV_U3PHYA_BASE
			reg_val &= ~0x3f;
			reg_val = reg_val | ((efuse_reg_val & (0x3f0000))>>16); //RG_SSUSB_IEXT_INTR_CTRL
			vIO32Write4B(0x100D0B34, reg_val);
	
			reg_val = u4IO32Read4B(0x100D0910); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val = reg_val | 0x80000000 | ((efuse_reg_val & (0x7c00000)) << 2); //rg_ssusb_tx_impsel
			vIO32Write4B(0x100D0910, reg_val);
	
			reg_val = u4IO32Read4B(0x100D0914); //SSUSB_SIFSLV_U3PHYD_BASE
			reg_val &= ~0x9f000000;
			reg_val = reg_val | 0x80000000 | ((efuse_reg_val & 0xf8000000)>>3); //rg_ssusb_rx_impsel
			vIO32Write4B(0x100D0914, reg_val);
		}
#endif
#if defined(CC_MT5893) //for PTPOD efuse value
    vIO32Write4B(0x10066290, u4IO32Read4B(0x10063090));
    vIO32Write4B(0x10066420, u4IO32Read4B(0x10063094));
    vIO32Write4B(0x10066424, u4IO32Read4B(0x10063098));
    vIO32Write4B(0x10066428, u4IO32Read4B(0x1006309c));
    vIO32Write4B(0x1006642c, u4IO32Read4B(0x100630b0));
#endif

    // 12. Set Postscaler output panel timing to output stage
    #if 0//current MJC cannot enable MJC or disable MJC by define, need basd on 4K panel or not
    #if defined(CC_SUPPORT_MJC)
    vIO32WriteFldAlign(OSTG_01, 1, REG_MJC_IN_EN);
    #else
    vIO32WriteFldAlign(OSTG_01, 0, REG_MJC_IN_EN);
    #endif
    #endif

    vIO32WriteFldAlign(OSTG_01, 1, R_BYPASS_OD);

    // 13. Set PTP pll source
    // PTP_ck clock selection: 480/8=60MHz
    // 0: xtal_ck
    // 1: usbpll_d8
    // 2: sawlesspll_d8
    // 3: syspll_d16
    // PTP_SLOW_ck clock selection: 24MHz/512=48KHz
    // 2'b00:	xtal_ck
    // 2'b01:  xtal_d512
    // 2'b10:	xtal_d1024
    // 2'b11:  xtal_d2048
    vIO32WriteFldAlign(CKGEN_REG_PTP_CKCFG, 0x1, FLD_PTP_CK_SEL);
    vIO32WriteFldAlign(CKGEN_REG_PTP_SLOW_CKCFG, 0x1, FLD_PTP_SLOW_CK_SEL);

    //GPU setting
	vIO32WriteFldAlign(CKGEN_REG_G3D_CORE_CKCFG, 1, FLD_G3D_CORE_SEL); //g3dpll_d1_ck
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x1f, FLD_MFG_RESET); //GPU MFG reset, toggle
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x0, FLD_MFG_RESET);
	vIO32WriteFldAlign(CKGEN_REG_G3D_CFG1, 0x1f, FLD_MFG_RESET);
    //socif sync  delay 2T -> 3T
    vIO32WriteFldMulti(GPU_MFG_ASYNC_CON, P_Fld(2, FLD_GPU_MFG_SOCIF_SLV_SEL) | P_Fld(2, FLD_GPU_MFG_SOCIF_MST_SEL));
    //Disable GPU ultra mode 
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_PR_EN);
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_LL_EN);
    vIO32WriteFldAlign(GPU_MFG_LATCTRL_CON_00, 0x0, FLD_GPU_MFG_ULTRA_WR_EN);
    //disable GPU DCM
    vIO32WriteFldAlign(GPU_MFG_GLOBAL_CON, 0x1, FLD_GPU_MFG_PSEL_M_GLOBAL_CON_OPT);
}

#ifdef CC_ARM_GIC
static unsigned int mt53xx_irq_pol[] = {
0xffffffff, 0xffffffff,
0x000000ff, 0x00000000,
0x00000010, 0x00000000,
0xc0000000, 0xfc01003f,
0x0007ff7f, 0xffffffff,
0xffffffff, 0xfff00000,
0x01c03fff, 0x000003fc
};

void GIC_Interrupt_Polarity(void)
{
	int i = 0, size = sizeof(mt53xx_irq_pol) / sizeof(mt53xx_irq_pol[0]);

	for (i = 0; i < size; i++)
	{
		if (i < 8)
		{
			MCU_WRITE32(REG_MCU_INT_POL_CTL0 + (i * 4), mt53xx_irq_pol[i]);
		}
		// polarity register is over 8 sets
		else
		{
			MCU_WRITE32(REG_MCU_INT_POL_CTL8 + ((i - 8) * 4), mt53xx_irq_pol[i]);
		}
	}
}
#endif

void CHIP_BimInit(void)
{
#ifdef CC_MTK_PRELOADER
    if (!CHIP_IsRemap())
    {
        // set bus access timeout to avoid register access halting CPU.
        BIM_WRITE32(REG_RW_TIMEOUT_CTRL, TO_FLASH_CNT | TO_DRAM_CNT | TO_IOBUS_CNT | TO_FLASH_EN | TO_DRAM_EN | TO_IOBUS_EN);
        //BIM_WRITE32(REG_RW_BTIME, TO_IORW | TO_IOWW);

        // Disable irq & fiq
        BIM_WRITE32(REG_RW_IRQEN, 0);
        BIM_WRITE32(REG_RW_FIQEN, 0);
        BIM_WRITE32(REG_RW_IRQCL, 0xffffffff);
        BIM_WRITE32(REG_RW_FIQCL, 0xffffffff);

        #ifndef CC_LOAD_UBOOT // only cpu0 is working in nucleus
        BIM_CpuDie(1);
        BIM_CpuDie(2);
        BIM_CpuDie(3);
        #endif

        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU1_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU1_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU2_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU2_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(MCUSYS_CONFIG_BASE, CPU3_SPMC0, (IO_READ32(MCUSYS_CONFIG_BASE, CPU3_SPMC0)) | ((0x1 << 1)));
        IO_WRITE32(BIM_BASE, BANK_SECURE_ENABLE1, (IO_READ32(BIM_BASE, BANK_SECURE_ENABLE1)) & (~(0x1 << 9)));

	#ifdef CC_ARM_GIC
	GIC_Interrupt_Polarity();
	#endif
    }
#else
    #ifdef CC_ARM_GIC
    GIC_Init();
    #endif
#endif // CC_MTK_PRELOADER
}

void CHIP_1stBootRegInit(void)
{
#ifndef CC_MTK_PRELOADER
    // Initial timer, then can use HAL_Delay_us()
    VERIFY(HAL_InitTimer());
#endif /* CC_MTK_PRELOADER */
}
void CHIP_Reboot(void)
{
    HAL_WRITE32(PDWNC_WDTCTL, 0x1); // PDWNC_WRITE32(REG_RW_WATCHDOG_EN, 1);//vIO32WriteFldAlign(PDWNC_WDTCTL, 0x1, FLD_WDT0E);
    HAL_WRITE32(PDWNC_WDT0, 0xffffffff); // PDWNC_WRITE32(REG_RW_WATCHDOG_TMR, 0xffffff00);//vIO32Write4B(PDWNC_WDT0, 0xffffff00);
    ASSERT(0);
}
