/* SPDX-License-Identifier: LGPL-2.1-only OR BSD-3-Clause */
/******************************************************************************
 *
 * This file is provided under a dual license.  When you use or
 * distribute this software, you may choose to be licensed under
 * version 2.1 of the GNU Lesser General Public License
 * ("LGPLv2.1 License") or BSD License.
 *
 * LGPLv2.1 License
 *
 * Copyright(C) 2019 MediaTek Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2.1 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See https://www.gnu.org/licenses/old-licenses/lgpl-2.1.html for more
 * details.
 *
 * BSD LICENSE
 *
 * Copyright(C) 2019 MediaTek Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  * Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

/*-----------------------------------------------------------------------------
 *
 * Description:
 *         The main routine of boot preloader
 *
 *---------------------------------------------------------------------------*/

#include "x_typedef.h"
#include "x_bim.h"
#include "x_hal_5381.h"
#include "x_ldr_env.h"
#include "x_hal_arm.h"
#include "x_dram.h"
#include "hw_pdwnc.h"
#include "loader_imghdr.h"
#include "gcpu_if.h"
#include "chip_txt.h"

//---------------------------------------------------------------------
// External declarations
//---------------------------------------------------------------------
#ifndef CC_LOADER_MEM_OFFSET
#define CC_LOADER_MEM_OFFSET (DRAM_START + 0xd00000)
#endif /* CC_LOADER_MEM_OFFSET */

EXTERN void *x_memcpy(void *dest, const void *src, UINT32 z_size);
EXTERN void *x_memcmp(void *dest, const void *src, UINT32 z_size);

EXTERN UINT32 INT_Reset;
EXTERN UINT32 INT_Vectors;

#if !defined(__GCC__)
EXTERN UINT32 Image$$DRAMTAG$$Limit;
#else  /*__KERNEL__*/
EXTERN UINT32 __bss_start;
EXTERN UINT32 __dramtag_end;
#endif /*__KERNEL__*/

//---------------------------------------------------------------------
// Internal declarations
//---------------------------------------------------------------------
#if defined(CC_NAND_BOOT)
extern BOOL _fgNandInit;
void CHIP_NANDCopyToDRAMLZHS(UINT32 *pu4DstBuf, UINT32 u4ByteOffset, UINT32 u4Len);
#endif /*CC_NAND_BOOT*/

#if defined(CC_EMMC_BOOT)
extern BOOL _fgMsdcInit;
INT32 CHIP_eMMCCopyToDRAM(UINT32 *pu4DstBuf, UINT64 u8ByteOffset, UINT32 u4Len);
#endif /*CC_EMMC_BOOT*/

//---------------------------------------------------------------------
// Private functions
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// Public functions
//---------------------------------------------------------------------
void __aeabi_unwind_cpp_pr1(void)
{
    return;
}

void __aeabi_unwind_cpp_pr0(void)
{
    return;
}

VOID LinkSymbol(UINT32 fgDisplay)
{
    if (fgDisplay)
    {
        CHIP_DisplayHex(INT_Reset);
        CHIP_DisplayHex(INT_Vectors);
    }
}

VOID ShowHex(const CHAR *pu1Str, UINT32 u4Hex)
{
    CHIP_DisplayString(pu1Str);
    CHIP_DisplayHex(u4Hex);
    CHIP_DisplayChar('\n');
}

#define PRELOADER_END   (SRAM_SECURE_BASE + 0x10000)
#define DRAM_BUFFER     (DRAM_START + 0x100000)
#define IMAGE_HEADER    (0x40)
#define LZHS_HEADER     (0x10)
#define LOADER_PADDING  (0x40)
VOID CMain()
{
    UINT32 u4BSSLimit = PRELOADER_MAX_SIZE;
    UINT32 u4ImageAddr, u4PlainSize, u4LzhsSize, u4Ret, u4DstAddr;
    UINT8 u1Checksum;
    PFN_IMAGE pfnImage;
    BOOL fgIsLzhs = TRUE;
    LDR_ENV_T *prLdrEnv = (LDR_ENV_T *)CC_LDR_ENV_OFFSET;

    LinkSymbol(0);
#ifdef CC_DUALLOADER_SUPPORT
    if (BIM_READ32(REG_RW_GPRDW6) == DUALLOADER_B) {u4BSSLimit += SECOND_BANK_OFFSET;}
#endif
#if defined(CC_NAND_BOOT)
    CHIP_DisplayString("NAND boot\n");
    _fgNandInit = FALSE;
    CHIP_NANDCopyToDRAMLZHS((UINT32 *)DRAM_BUFFER, u4BSSLimit, IMAGE_HEADER + LZHS_HEADER);
    u4ImageAddr = DRAM_BUFFER + IMAGE_HEADER;
#elif defined(CC_EMMC_BOOT)
    CHIP_DisplayString("EMMC boot\n");
    _fgMsdcInit = FALSE;
    UINT32 u4eMMCRead = CHIP_eMMCCopyToDRAM((UINT32 *)DRAM_BUFFER, u4BSSLimit, IMAGE_HEADER + LZHS_HEADER);
    if (0 != u4eMMCRead) {ShowHex("EMMC Read error:", u4eMMCRead);}
    u4ImageAddr = DRAM_BUFFER + IMAGE_HEADER;
#else
    CHIP_DisplayString("NOR boot\n");
    u4ImageAddr = u4BSSLimit + IMAGE_HEADER + PBI_A_BASE;
#endif
#ifdef CC_SECURE_BOOT_SCRAMBLE
    GCPU_LoaderInit(0);
    GCPU_SetAesKey((UINT8 *)(PRELOADER_END - 0x400), KEY_BIT_LEN_128, (UINT8 *)(PRELOADER_END - 0x3f0));
    GCPU_AesDecrypt(u4ImageAddr, u4ImageAddr, LZHS_HEADER);
#endif // CC_SECURE_BOOT_SCRAMBLE

    ShowHex("LZHS addr:", u4ImageAddr);
    u4PlainSize = *((UINT32 *)u4ImageAddr);
    ShowHex("LZHS size:", u4PlainSize);
    u4LzhsSize = *((UINT32 *)(u4ImageAddr + 4));
    if (u4PlainSize == u4LzhsSize)
    {
        fgIsLzhs = FALSE;
        u4DstAddr = CC_LOADER_MEM_OFFSET - (IMAGE_HEADER + LZHS_HEADER * 2);
    }
    else
    {
        fgIsLzhs = TRUE;
        ShowHex("LZHS size:", u4LzhsSize);
        u1Checksum = (*((UINT8 *)(u4ImageAddr + 8)));
        ShowHex("LZHS checksum:", (UINT32)u1Checksum);
        u4DstAddr = DRAM_BUFFER;
    }
    // total size of loader for u4LdrVerOffset
#ifdef CC_SECURE_BOOT_V3
    u4LzhsSize = ((u4LzhsSize + 0x10) & (~0xf)) + LOADER_PADDING + 0x300;
#endif // CC_SECURE_BOOT_V3
    if (u4LzhsSize > LOADER_MAX_SIZE) {u4LzhsSize = LOADER_MAX_SIZE;} //limit size of loader

    // store loader size, address for loader usage
    prLdrEnv->u4LoaderPlainSize = u4PlainSize;
    prLdrEnv->u4LoaderLzhsSize = u4LzhsSize;
    prLdrEnv->u4LoaderLzhsAddress = u4BSSLimit + IMAGE_HEADER + LZHS_HEADER;

    u4LzhsSize += IMAGE_HEADER + LZHS_HEADER;   //+offset
#if defined(CC_NAND_BOOT)
    CHIP_NANDCopyToDRAMLZHS((UINT32 *)u4DstAddr, u4BSSLimit, u4LzhsSize);
#elif defined(CC_EMMC_BOOT)
    u4eMMCRead = CHIP_eMMCCopyToDRAM((UINT32 *)u4DstAddr, u4BSSLimit, u4LzhsSize);
    if (0 != u4eMMCRead) {ShowHex("EMMC Read error:", u4eMMCRead);}
#endif
#ifdef CC_SECURE_BOOT_V3
    u4LzhsSize -= 0x300;                        //-signature
    if (BIM_IS_SECURE_BOOT)
    {
        EXTERN BOOL BIM_VerifyImg(UINT8 *pu1Img, UINT32 u4Size, UINT8 *pu1PubKey);
        BIM_VerifyImg((UINT8 *)(u4DstAddr + u4LzhsSize + 256), 256, (UINT8 *)(PRELOADER_END - 512));
        BIM_VerifyImg((UINT8 *)(u4DstAddr + IMAGE_HEADER), u4LzhsSize - IMAGE_HEADER, (UINT8 *)(u4DstAddr + u4LzhsSize + 256));
    }
    x_memcpy((void *)prLdrEnv->au4CustKey, (void *)(u4DstAddr + u4LzhsSize + 256), sizeof(prLdrEnv->au4CustKey));
#endif // CC_SECURE_BOOT_V3
#ifdef CC_SECURE_BOOT_SCRAMBLE
    GCPU_AesDecrypt(u4DstAddr + IMAGE_HEADER, u4DstAddr + IMAGE_HEADER, u4LzhsSize - IMAGE_HEADER);
#endif // CC_SECURE_BOOT_SCRAMBLE
#if defined(CC_SECURE_BOOT_SCRAMBLE) || defined(CC_DLM_V2)
    x_memcpy((void *)prLdrEnv->aes_key2, (void *)(u4DstAddr + u4LzhsSize - 64), sizeof(prLdrEnv->aes_key2));
    x_memcpy((void *)prLdrEnv->aes_iv2, (void *)(u4DstAddr + u4LzhsSize - 48), sizeof(prLdrEnv->aes_iv2));
#endif // CC_SECURE_BOOT_SCRAMBLE || CC_DLM_V2
#ifdef CC_LOADER_CONSOLE_KEY_SUPPORT
    x_memcpy((void *)prLdrEnv->console_key, (void *)(u4DstAddr + u4LzhsSize - 32), sizeof(prLdrEnv->console_key));
    x_memcpy((void *)prLdrEnv->console_iv, (void *)(u4DstAddr + u4LzhsSize - 16), sizeof(prLdrEnv->console_iv));
#endif // CC_LOADER_CONSOLE_KEY_SUPPORT

    if (!fgIsLzhs)
    {
        CHIP_DisplayString("plaintext boot\n");
    }
    else
    {
        CHIP_DisplayString("LZHS begin \n");
        u4Ret = LZHS_Dec(u4ImageAddr + LZHS_HEADER, CC_LOADER_MEM_OFFSET - LZHS_HEADER, u4PlainSize, &u1Checksum);
        if (u4Ret != 0)
        {
            CHIP_DisplayString("LZHS failed\n");
#ifdef CC_DUALLOADER_SUPPORT
            if (BIM_READ32(REG_RW_GPRDW6) != DUALLOADER_B)
            {
                EXTERN void CHIP_Reboot(void);
                CHIP_DisplayString("\nTrying 2nd bank...");
                CHIP_Reboot();
            }
#endif
            HaltSystem();
        }
        CHIP_DisplayString("LZHS ok\n");
    }
#ifdef CC_LOAD_UBOOT
    u4LzhsSize = *(UINT32 *)(CC_LOADER_MEM_OFFSET - LZHS_HEADER); //hot-boot preloader offset
    if (u4PlainSize - u4LzhsSize - LZHS_HEADER <= 0x4000)
    {
        x_memcpy((void *)SRAM_PDWNC_BASE, (void *)(CC_LOADER_MEM_OFFSET + u4LzhsSize), u4PlainSize - u4LzhsSize - LZHS_HEADER);
        if (x_memcmp((void *)SRAM_PDWNC_BASE, (void *)(CC_LOADER_MEM_OFFSET + u4LzhsSize), u4PlainSize - u4LzhsSize - LZHS_HEADER) == 0)
        {
            vIO32Write4B(PDWNC_TZ_RESERVED_0, SRAM_PDWNC_BASE);
            vIO32Write4B(PDWNC_TZ_RESERVED_1, 0x1);
            vIO32WriteFldAlign(PDWNC_CONCFG1, 0x1, FLD_RG_HOTBOOT_LOG);
        }
        else
        {
            CHIP_DisplayString("Hotboot preloader error!\n");
        }
    }
#endif // CC_LOAD_UBOOT
    BIM_SetTimeLog(1);
    pfnImage = (PFN_IMAGE)CC_LOADER_MEM_OFFSET;
    pfnImage();
}
